% Please cite the following two papers if you are using this code or the
% accompanying RF-data 

% [1] Mirzaei, M., Asif, A., Rivaz, H., Combining Total Variation Regularization 
% with Window-based Time Delay Estimation in Ultrasound Elastography., 
% IEEE Trans. Medical Imaging, 2019.
% [2] H. Rivaz, Boctor, E., Foroughi, P., Zellars, R., Fichtinger, G., Hager, G., 
% Ultrasound Elastography: a Dynamic Programming Approach, IEEE Trans. Medical Imaging, 2008.


%%  ------------------------------------------------------- %
%   -------------- loading RF Data ------------------------ %
%   ------------------------------------------------------- %
load '0_both_apodization.mat'
Im2=Im5;
load '1_both_apodization.mat'
Im1=Im5;

maxIm=max(Im2(:));
Im2=(Im2/maxIm);
Im1=(Im1/maxIm);  
%% ------------------------------------------------------- %
%  ------------- set See_B_mode to see B-mode ------------ %
%  ------------------------------------------------------- %
See_B_mode=1;
if See_B_mode
    BMODE1=log(abs(hilbert(Im1))+.01);
    figure,imagesc([0 37.1],[0 1341*1540/80000],BMODE1);colormap(gray), colorbar
    
    BMODE2=log(abs(hilbert(Im2))+.01);
    figure,imagesc([0 37.1],[0 1341*1540/80000],BMODE2);colormap(gray), colorbar
end
%% ------------------------------------------------------- %
%  ---------- initial displacements using DP ------------- %
%  ------------------------------------------------------- %
%%%%%%%%%%%%%%%%% RCF_AM2D, fast version of DP 
IRF = [-35 35]; % Maximum allowed disparity in axial D
IA = [-4 4]; % Maximum allowed disparity in lateral D
midA = 50;
alfa = 5; %axial regularization
beta = 10; %lateral regularization
gamma = 0.05; %lateral regularization 
T = .2; % threshold for IRLS
xx = 1; % to compensate for attenuation
alfa_DP = 0.02; % DP regularization weight
[ax0, lat0, ~] = RCF_AM2D(double(Im2), double(Im1), IRF, IA, midA, alfa_DP, alfa, beta, gamma, T, xx);
ax0 = ax0(41:end-40, 16:end-15);
lat0 = lat0(41:end-40, 16:end-15);
Im1 = Im1(41:end-40, 16:end-15);
Im2 = Im2(41:end-40, 16:end-15);

%%%%%%%%%%%%%%%%%%%%%%%%%% Original DP
% r1=0.1; 
% 
% dmaxa=-50; %% maximum search range in axial direction
% dminl=-1; %% maximum search range in lateral direction (left side)
% dmaxl=1; %% maximum search range in lateral direction (right side)
% 
% [ax0,lat0]=Dynamic2Copy(double(Im2), double(Im1),r1,dmaxa,dminl,dmaxl);
% [m,n]=size(Im1);
% rffirst=abs(dmaxa);
% rfend=m;
% firstline=abs(dminl)+1;
% lastline=n-abs(dmaxl);
% Im1=Im1(rffirst:rfend,firstline:lastline);
% Im2=Im2(rffirst:rfend,firstline:lastline);

%% ------------------------------------------------------- %
%  ---------- regularization coefficients ---------------- %
%  ------------------------------------------------------- % 
alph1=40; %% regularize smoothneth in axial direction 
alph2=0.5;  %% regularize smoothneth in lateral direction 
betta1=20;  %% it is better to be set as 0.5*alph1;
betta2=0.25;   %% it is better to be set as 0.5*alph2;
Tr1=0.02; %% \lambda_1 &  \lambda_3 it is better to be set as 0.02
Tr2=0.02; %% \lambda_2 &  \lambda_4 it is better to be set as 0.02

%% ------------------------------------------------------- %
%  ---------- Window sizes for data term ----------------- %
%  ------------------------------------------------------- % 
w1=3;   %% (2*w_a+1), so w1 should be odd
w2=3;    %% (2*w_l+1), so w2 should be odd
%% ------------------------------------------------------- %
%  ---------- Run OVERWIND ------------------------------- %
%  ------------------------------------------------------- %

[Axial, Lateral] = OVERWIND (ax0, lat0, Im2, Im1, alph1, alph2, betta1, betta2,Tr1,Tr2,w1,w2);
figure,  imagesc(Axial),colormap(jet), colorbar, title('Axial Disaplcement');
figure,  imagesc(Lateral),colormap(jet), colorbar, title('Lateral Disaplcement');
%% ------------------------------------------------------- %
%  ---------- Plot disploacement and strain -------------- %
%  ------------------------------------------------------- % 
wDIff = 63;
strain1 = LSE(-Axial,wDIff);


figure,  imagesc([-10,10],[5,25],strain1,[0.00002 0.012]),colormap(gray), colorbar, title('Axial Strain');
